#include <Servo.h>

int PIN_RCV = 2; //Вход приемника
int PIN_SRV_DOOR = 3;  // Выход сервы люка шасси
int PIN_SRV_GEAR = 4;  // Выход сервы ретракта
int PIN_LED = 5; // выход  светодиода мигает во время работы.
int dmin = 1050; // Минимальная ширина импульса с приемника
int dmax = 1950; // Максимальная
#define MODE_OFF 0
#define MODE_ON 1
#define MODE_PAUSE 2
int mode = MODE_PAUSE;
int DOOR_MIN = 30; // Минимальный угол сервы люка
int DOOR_MAX = 150; // Максимальный угол сервы люка
int DOOR_SPEED = 2; //  Скорость сервы люка
int DOOR_REVERSE = 1; // Реверс сервы люка 0 или 1
int GEAR_MIN = 30; // Минимальный угол сервы ретракта
int GEAR_MAX = 150; // Максимальный угол сервы ретракта
int GEAR_SPEED = 2; //  Скорость сервы ретракта
int GEAR_REVERSE = 0; // Реверс сервы ретракта 0 или 1
int PAUSE = 10; // Пауза между открытием люка и выпуском шасси

#define PHASE_DOOR 0
#define PHASE_PAUSE 1
#define PHASE_GEAR 2

int phase = PHASE_DOOR;
int pause_cnt = 0;

#define LED_SPEED 5
int led_cnt = 0;
int led_cnd = 0;

int sd = 0;
int sg = 0;
Servo srv_door;
Servo srv_gear;


void setup() {
  pinMode( PIN_RCV, INPUT);
  srv_gear.attach(PIN_SRV_GEAR);
  sg = GEAR_MIN;
  if(GEAR_REVERSE) srv_gear.write( GEAR_MAX-sg +GEAR_MIN );
  else srv_gear.write(sg);
  delay(2000);
  srv_door.attach(PIN_SRV_DOOR);
  sd = DOOR_MIN;
  if(DOOR_REVERSE) srv_door.write( DOOR_MAX-sd +DOOR_MIN );
  else srv_door.write(sd);
  pinMode( PIN_LED, OUTPUT);
//  Serial.begin(9600);
}

void led_flash(){
  if( led_cnt >= LED_SPEED){
    led_cnt = 0;
    led_cnd = 1-led_cnd;
  }
  led_cnt++;
  if(led_cnd==0)digitalWrite(PIN_LED, HIGH);
  else digitalWrite(PIN_LED, LOW);    
}
void led_off(){
  digitalWrite(PIN_LED, HIGH);
}

void loop() {
  int d = pulseIn(PIN_RCV, HIGH, 25000);
  if(d==0){ delay(5);  return; }
  if(d<dmin) dmin = d;
  if(d>dmax) dmax = d;
  int p = map(d, dmin, dmax, 0, 100);
  if(p<=33){ //Шасси убрать
    mode = MODE_OFF;
//    Serial.print(" mode: OFF");
  }
  else if(p>=67){ //Шасси выпустить
    mode = MODE_ON;
//    Serial.print(" mode: ON");
  }  
  else{ //Пауза
    mode = MODE_PAUSE;
//    Serial.print(" mode: PAUSE");
  }
//  Serial.print(" sd:");
//  Serial.print( sd );
//  Serial.print(" sg:");
//  Serial.print( sg );

  if(phase == PHASE_DOOR){
//    Serial.print(" PHASE_DOOR");
    if(mode == MODE_ON){
      if(sd >= DOOR_MAX){ 
        phase = PHASE_PAUSE; 
//        Serial.println(" D->P");
        return;
      }
      sd = sd + DOOR_SPEED; 
      if( sd > DOOR_MAX) sd = DOOR_MAX;
      if(DOOR_REVERSE) srv_door.write( DOOR_MAX-sd +DOOR_MIN );
      else srv_door.write(sd);
      led_flash();
//      Serial.println(" D->");
      return;
    }
    else if(mode == MODE_OFF){
      if(sd <= DOOR_MIN) {
        led_off();
        delay(5); 
//        Serial.println(" DONE"); 
        return;
      }
      sd = sd - DOOR_SPEED;
      if(sd < DOOR_MIN )sd = DOOR_MIN;
      if(DOOR_REVERSE) srv_door.write( DOOR_MAX-sd +DOOR_MIN );
      else srv_door.write(sd);
      led_flash();
//      Serial.println(" <-D");
      return;
    }
    else{
      led_off();
//      Serial.println(" PAUSE");
      delay(5);  return;
    }
  }

  if(phase == PHASE_GEAR){
//    Serial.print(" PHASE_GEAR");
    if(mode == MODE_ON){
      if(sg >= GEAR_MAX){ 
        led_off();
        delay(5); 
//        Serial.println(" DONE");  
        return;
      }
      sg = sg + GEAR_SPEED; 
      if( sg > GEAR_MAX) sg = GEAR_MAX;
      if(GEAR_REVERSE) srv_gear.write( GEAR_MAX-sg +GEAR_MIN );
      else srv_gear.write(sg);
      led_flash();
//      Serial.println(" G->");  
      return;
    }
    else if(mode == MODE_OFF){
      if(sg <= GEAR_MIN) {
        phase = PHASE_PAUSE; 
//        Serial.println(" G>P");  
        return;
      }
      sg = sg - DOOR_SPEED;
      if(sg < GEAR_MIN )sg = GEAR_MIN;
      if(GEAR_REVERSE) srv_gear.write( GEAR_MAX-sg +GEAR_MIN );
      else srv_gear.write(sg);
      led_flash();
//      Serial.println(" <-G");  
      return;
    }
    else{
      led_off();
//      Serial.println(" PAUSE");
      delay(5);  
      return;
    }
  }
  if(phase == PHASE_PAUSE){
//    Serial.print(" PHASE_PAUSE");
    if(pause_cnt >= PAUSE){
      pause_cnt = 0;
      if(mode == MODE_OFF){
        phase = PHASE_DOOR;
//        Serial.println(" P->D");  
      }
      if(mode == MODE_ON) {
        phase = PHASE_GEAR;
//         Serial.println(" P->G");  
      }      
      delay(5);
      return;
    }
    pause_cnt++;
    led_flash();
//    Serial.println(" P...");  
    delay(50);
    return;
  }
 
//  Serial.println("!!!!!!!!!!!!");  
  delay(1);   
}
